/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.portals.gems.dojo;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;

import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.lang.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.web.portlet.DispatcherPortlet;


/**
 * Abstract DOJO portlet for inserting in cross context dojo widget includes
 * 
 * @author <a href="mailto:smilek@apache.org">Steve Milek</a>
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a> 
 * @version $Id$
 */
public class DojoSpringMVCPortlet extends DispatcherPortlet
{
    /*
     * Class specific logger.
     */
    private final static Logger log = LoggerFactory.getLogger(DojoSpringMVCPortlet.class);

	protected static final String CRLF = "\r\n";
    
    protected static final String DOJO_JS_URL_INIT_PARAM = "dojo.js.url";
    protected static final String DOJO_REQUIRES_CORE_INIT_PARAM = "dojo.requires.core";
    protected static final String DOJO_REQUIRES_MODULES_INIT_PARAM = "dojo.requires.modules";
    
    private String dojoJsUrl;
    private String dojoRequiresCoreList = null;
    private String dojoRequiresAddOnList = null;
    
    protected String headerPage;
    
    /*
     * Portlet constructor.
     */
    public DojoSpringMVCPortlet() 
    {
        super();
    }
    
    /*
     * Portlet lifecycle method.
     */
    protected void initFrameworkPortlet() throws PortletException, BeansException
    {
        super.initFrameworkPortlet();

        // access jetspeed heaader resource component
        synchronized (this) 
        {
            this.headerPage = this.getInitParameter("HeaderPage");
            this.dojoJsUrl = getInitParameter(DOJO_JS_URL_INIT_PARAM);
            this.dojoRequiresCoreList = this.getInitParameter( DOJO_REQUIRES_CORE_INIT_PARAM );
            this.dojoRequiresAddOnList = this.getInitParameter( DOJO_REQUIRES_MODULES_INIT_PARAM );
        }
    }

    /* (non-Javadoc)
     * @see javax.portlet.GenericPortlet#doDispatch(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
     */
    protected void doRenderService(RenderRequest request, RenderResponse response) throws Exception
    {
        // dispatch normally
        super.doRenderService(request, response);
    }

    @Override
    protected void doHeaders(RenderRequest request, RenderResponse response)
    {
        DojoPortletHelper.enableDojo(response, this.dojoJsUrl);
        
        if (this.headerPage != null)
        {
            try
            {
                this.getPortletContext().getRequestDispatcher(this.headerPage).include(request, response);
            }
            catch (PortletException e)
            {
                throw new RuntimeException("Failed to include header page.", e);
            }
            catch (IOException e)
            {
                throw new RuntimeException("Failed to include header page.", e);
            }
        }
        
        if (this.dojoRequiresCoreList != null)
        {
            DojoPortletHelper.contributeDojoRequires(response, this.dojoRequiresCoreList);
        }
        
        if (this.dojoRequiresAddOnList != null)
        {
            DojoPortletHelper.contributeDojoRequires(response, this.dojoRequiresAddOnList);
        }
    }

    /**
     * Utility method to allow javascript head element contribution.
     * Be sure that this method can be invoked only in the {@link #doHeaders(RenderRequest, RenderResponse)} method
     * that you may override.
     * 
     * @param response
     * @param textContent
     * @return
     */
    protected boolean addJavascriptBlock(RenderResponse response, String textContent)
    {
        return addJavascriptElement(response, null, textContent);
    }

    /**
     * Utility method to allow javascript head element contribution.
     * Be sure that this method can be invoked only in the {@link #doHeaders(RenderRequest, RenderResponse)} method
     * that you may override.
     *
     * @param response
     * @param src
     * @return
     */
    protected boolean addJavascriptInclude(RenderResponse response, String src)
    {
        return addJavascriptElement(response, src, null);
    }
    
    /**
     * Utility method to allow javascript head element contribution.
     * Be sure that this method can be invoked only in the {@link #doHeaders(RenderRequest, RenderResponse)} method
     * that you may override.
     * 
     * @param response
     * @param src
     * @param textContent
     * @return
     */
    protected boolean addJavascriptElement(RenderResponse response, String src, String textContent)
    {
        if ((textContent != null && textContent.length() > 0) || (src != null && src.length() > 0))
        {
            DojoPortletHelper.contributeScript(response, "JavaScript", "text/javascript", src, textContent, null, null);
            return true ;
        }
        
        return false ;
    }
    
	// debugging

	protected void dumpAttributes(RenderRequest request)
	{
		Enumeration attrIter = request.getAttributeNames();
		log.info( "request-attributes:");
		
		while ( attrIter.hasMoreElements() )
		{
			Object attrNm = attrIter.nextElement();
			Object attrVal = request.getAttribute(ObjectUtils.toString(attrNm));
			String attrValDesc = ( attrVal instanceof String ) ? (String)attrVal : (( attrVal == null ) ? "null" : attrVal.getClass().getName() );
			log.info( "   key=" + ObjectUtils.toString(attrNm,"null") + " value=" + attrValDesc);
		}
	}
	
	protected void dumpSession(RenderRequest request)
	{
		Enumeration attrIter = request.getPortletSession().getAttributeNames();
		log.info( "session-attributes:");
		
		while ( attrIter.hasMoreElements() )
		{
			Object attrNm = attrIter.nextElement();
			Object attrVal = request.getPortletSession().getAttribute(ObjectUtils.toString(attrNm));
			String attrValDesc = ( attrVal instanceof String ) ? (String)attrVal : (( attrVal == null ) ? "null" : attrVal.getClass().getName() );
			log.info( "   key=" + ObjectUtils.toString(attrNm,"null") + " value=" + attrValDesc);
		}
	}
	
	protected void dumpNameValue( Map m  )
	{
		if ( m == null )
		{
			log.info( "   <null>" );
			return;
		}
		
		if ( m.size() == 0 )
		{
			log.info( "   <empty>" );
			return;
		}
		
		Iterator entryIter = m.entrySet().iterator();
		
		while ( entryIter.hasNext() )
		{
			Map.Entry e = (Map.Entry)entryIter.next();
			Object eKey = e.getKey();
			Object eVal = e.getValue();
			String eKeyDesc = ( eKey instanceof String ) ? (String)eKey : (( eKey == null ) ? "null" : eKey.getClass().getName() );
			String eValDesc = ( eVal instanceof String ) ? (String)eVal : (( eVal == null ) ? "null" : eVal.getClass().getName() );
			log.info( "   key=" + eKeyDesc + " value=" + eValDesc);
		}
	}
	
}
